/**
 * P4
 * http://snarfed.org/space/p4
 *
 * Copyright 2001-2004
 * Ken Ashcraft <kashcraft+p4@gmail.com>
 * Ryan Barrett <p4@ryanb.org>
 * Maulik Shah <p4@maulik.net>
 * Nathan Stoll <nathanstoll+p4@gmail.com>
 *
 * @file msg_lka_ack.cpp
 *
 * Defines the cmsg_lka_ack class, which is derived from the abstract cmsg
 * class. The cmsg_lka_ack class is the class that LKA_ACK messages
 * instantiate. For more information, see protocol.txt, msg.h, and protocol.h.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "msg_lka_ack.h"
#include "debug.h"
#include "p4error.h"

/*****************************************************************************
 * INITIALIZE STATIC MEMBERS
 *****************************************************************************/

pthread_mutex_t cmsg_lka_ack::received_mtx = PTHREAD_MUTEX_INITIALIZER;
pthread_cond_t cmsg_lka_ack::received_cond = PTHREAD_COND_INITIALIZER;
bool cmsg_lka_ack::received = false;

/*****************************************************************************
 * PUBLIC METHODS
 *****************************************************************************/

cmsg_lka_ack::cmsg_lka_ack() : cmsg(0, 0, 0)
{ 
  data = reinterpret_cast<char*> (clka::instance().package(hdr.data_len));
  host_data_len = hdr.data_len;
  hdr.set_type(kp4_lka_ack);
}

/*****************************************************************************
 * PROTECTED METHODS
 *****************************************************************************/

bool cmsg_lka_ack::virt_process() const
{
  // should make sure that actual data size is the same as what the msg says.

  uint ret = clka::instance().populate(reinterpret_cast<p4_lka_ack*>(data));

  if (ret != kerr_success)
    cdebug::instance() << acquire("msg_debug")
                       << "Couldn't populate lka. " << "Err # " << ret
                       << release();

  pthread_mutex_lock(&received_mtx);
  received = (ret == kerr_success);
  // we use signal cause only one thread can wait at a time
  pthread_cond_signal(&received_cond); 
  pthread_mutex_unlock(&received_mtx);
  
  return true;
}


/*****************************************************************************
 * PRIVATE METHODS
 *****************************************************************************/
void cmsg_lka_ack::convert_byteorder(cmsg::short_fn sfn, cmsg::long_fn lfn)
{
  cmsg::convert_byteorder(sfn, lfn);

  // convert lka_ack
  assert(data);
  p4_lka_ack *lka_ack = reinterpret_cast<p4_lka_ack *>(data);
  lka_ack->num_entries = lfn(lka_ack->num_entries);

  // get num_plugins in host order (this looks backwards because this is called
  // from cmsg::hton or ntoh, and byte_order hasn't been flipped yet)
  ulong num_entries = ((byte_order == ehost) ?
              ntohl(lka_ack->num_entries) : lka_ack->num_entries);
  // convert lka entries
  lka_entry *entries = reinterpret_cast<lka_entry *>(lka_ack + 1);
  for (uint i = 0; i < num_entries; i++) {
    entries[i].addr = lfn(entries[i].addr);
    entries[i].port = sfn(entries[i].port);
    entries[i].num_plugins = sfn(entries[i].num_plugins);
  }
}
